function love.load()
	version = "v1.1"
	love.window.setTitle("SquareZ "..version)
	windowWidth = 800
	windowHeight = 600
	love.window.setMode(windowWidth,windowHeight,{fullscreen=false, resizable=true})
	
	bgColor = {50,125,255}
	txtColor = {0,0,0}
	tileColor = {100,255,255}
	
	debugFont = love.graphics.setNewFont(14)
	smallFont = love.graphics.setNewFont(20)
	normalFont = love.graphics.setNewFont(24)
	bigFont = love.graphics.setNewFont(32)
	
	fullscreen = false
	hasWon = false
	showDebug = false
	hintStyle = 0
	
	tM, tS = 0, 0
	mx, my = 0, 0
	
	gridSize = 5
	
	getSize()
	gridReset()
	
	love.graphics.setBackgroundColor(bgColor)
	love.graphics.setColor(txtColor)
end

function love.update(dt)	
	if hasWon == false then tS = tS + dt end
	getSize()
end

function love.draw()	
	love.graphics.setFont(smallFont)
	love.graphics.print("F1 - Help\nF4 - Toggle Fullscreen",0,th-48)
	love.graphics.print("Grid Size: "..gridSize,gx,th-76)
	
	for i=1,gridSize do
		for j=1,gridSize do
			if player[i][j] == 1 then love.graphics.setColor(tileColor)
			else love.graphics.setColor(bgColor)
			end
			love.graphics.rectangle("fill",gx+(i-1)*(gs/gridSize),gy+(j-1)*(gs/gridSize),gs/gridSize,gs/gridSize)
		end
	end

	setBorder(gx,gy,gs,gs)
	
	showHint()
	
	if hasWon then
		love.graphics.setColor(tileColor)
		love.graphics.setFont(bigFont)
		love.graphics.print("YOU WIN!",0,96)
		love.graphics.setFont(smallFont)
		love.graphics.print("Click or press SPACE\nto play again.",0,128)
		love.graphics.setColor(txtColor)
	end
	
	love.graphics.setFont(normalFont)
	love.graphics.print("Elapsed Time",0,0)
	love.graphics.setFont(bigFont)
	love.graphics.print(string.format("%02d".."s",tS),0,24)
	
	if showDebug then Debug() end
end

function love.mousepressed(x,y,button)
	if button == "l" then
		if hasWon then gridReset() end
		changeGrid()
	end
end

function love.mousemoved(x,y)
	mx = x
	my = y
end

function getSize()
	tw = love.graphics.getWidth()
	th = love.graphics.getHeight()
	
	gs = 0
	if tw>th then gs = th-256
	else gs = tw-256 end
	gx = tw/2-(gs/2)
	gy = th/2-(gs/2)+48
end

function love.keypressed(key,isrepeat)
	if key == "escape" then
		love.event.quit()
	elseif key == " " then
		gridReset()
	elseif key == "f1" then
		local about = "ABOUT:\nProgramming by me, Jesse 'Xyore' Carrillo.\nOriginal concept not by me.\n\n"
		local howToPlay = "HOW TO PLAY:\nThe goal of this game is to fill the correct cells in the box according to the hints provided surrounding the box.\nEach number corresponds to the amount of shaded cells between each empty cell.\n\nHere is an example of a completed 3x3:\n\n           1\n     3    2    2\n1 [ X | X |     ]\n3 [ X |    | X ]\n3 [ X | X | X ]\n\n"
		local controls = "CONTROLS:\nClick a cell to fill or erase.\nSpace bar to reset.\nEsc to quit.\n'+' to increase and '-' to decrease grid size.\n1-9 to select grid size."
		love.window.showMessageBox("SquareZ "..version,about..howToPlay..controls)
	elseif key == "f4" then
		local width, height = love.window.getDesktopDimensions(1)
		local fullscreen,fstype = love.window.getFullscreen()
		if fullscreen then love.window.setMode(windowWidth,windowHeight,{fullscreen=false, resizable=true})
		else love.window.setMode(width,height,{fullscreen=true,fullscreentype="desktop"}) end
	elseif key == "`" then
		showDebug = not showDebug
	elseif key == "-" then
		if gridSize > 3 then gridSize = gridSize - 1 gridReset() end
	elseif key == "=" then
		if gridSize < 9 then gridSize = gridSize + 1 gridReset() end
	elseif key == "3" or key == "4" or key == "5" or key == "6" or key == "7" or key == "8" or key == "9" then
		gridSize = tonumber(key)
		gridReset()
	end
end

function gridReset()
	math.randomseed(os.time())
	hasWon = false
	tM, tS = 0, 0
	
	grid = {}	
	for i=1,gridSize do
		grid[i] = {}
		for j=1,gridSize do
			grid[i][j] = 0
		end
	end
	
	player = {}
	for i=1,gridSize do
		player[i] = {}
		for j=1,gridSize do
			player[i][j] = 0
		end
	end
	
	for i=1,gridSize do
		for j=1,gridSize do
			grid[i][j] = math.random(0,1)
		end
	end
	
	for i=1,gridSize do
		for j=1,gridSize do
			player[i][j] = 0
		end
	end
	
	numc, numr = getCellHints(grid)
end

function getCellHints(t)
	local numc = {}
	for i=1,gridSize do
		numc[i] = {}
	end
	
	local numr = {}
	for i=1,gridSize do
		numr[i] = {}
	end

	local tempc = 0	
	for i=1,gridSize do
		for j=1, gridSize do
			if t[i][j] == 1 then
				tempc = tempc + 1
				if j == gridSize then
					table.insert(numc[i],tempc)
					tempc = 0
				end
			else
				if tempc > 0 or (j == gridSize and #numc[i] == 0) then
					table.insert(numc[i],tempc)
					tempc = 0
				end
			end
		end	
	end
	
	local tempr = 0
	for i=1,gridSize do
		for j=1, gridSize do
			if t[j][i] == 1 then
				tempr = tempr + 1
				if j == gridSize then
					table.insert(numr[i],tempr)
					tempr = 0
				end
			else
				if tempr > 0 or (j == gridSize and #numr[i] == 0) then
					table.insert(numr[i],tempr)
					tempr = 0
				end
			end
		end	
	end
	
	return numc, numr
end

function setBorder(x,y,w,h,s)
	s = s or 0
	love.graphics.setColor(txtColor)
	love.graphics.setLineWidth(3)
	for i=1,gridSize-1 do
		love.graphics.line(x,y+i*(h/gridSize),x+w,y+i*(h/gridSize))
		for j=1,gridSize-1 do
			love.graphics.line(x+j*(w/gridSize),y,x+j*(w/gridSize),y+h)
		end
	end
	love.graphics.line(x,y,x+w,y,x+w,y+h,x,y+h,x,y)
end

function checkArea(checkX,checkY,x1,y1,x2,y2)
	if checkX >= x1 and checkX <= x2 and checkY >= y1 and checkY <= y2 then
		return true
	else
		return false
	end
end

function changeGrid()
	if hasWon == false then
		for i=1,gridSize do
			for j=1,gridSize do
				if checkArea(love.mouse.getX(),love.mouse.getY(),gx+(j-1)*(gs/gridSize),gy+(i-1)*(gs/gridSize),gx+j*(gs/gridSize),gy+i*(gs/gridSize)) then
					if player[j][i] == 0 then player[j][i] = 1
					else player[j][i] = 0
					end
				end
			end
		end
		checkWin()
	end
end

function checkWin()
	--[[
	local matchct = 0
	local matchsz = gridSize^2
	for i=1,gridSize do
		for j=1,gridSize do
			if player[i][j] == grid[i][j] then
				matchct = matchct + 1
			end
		end
	end
	if matchct == matchsz then hasWon = true end
	--]]
	
	local matches = 0
	local maxMatches = 0
	local matchc, matchr = getCellHints(player)
	for i=1,gridSize do
		for j=1,#numc[i] do
			maxMatches = maxMatches + 1
			if matchc[i][j] == numc[i][j] and matchc[i][j] ~= nil then matches = matches + 1 end
		end
	end
	
	for i=1,gridSize do
		for j=1,#numr[i] do
			maxMatches = maxMatches + 1
			if matchr[i][j] == numr[i][j] and matchr[i][j] ~= nil then matches = matches + 1 end
		end
	end
	
	if matches == maxMatches then hasWon = true end
end

function showHint()
	love.graphics.setFont(normalFont)
	love.graphics.setColor(txtColor)
	
	for i=1,gridSize do
		for j=1,#numc[i] do
			love.graphics.print(numc[i][#numc[i]+1-j],gx+((i-1)*(gs/gridSize))+(gs/gridSize/2-10),gy-32-((j-1)*32))
		end
	end
	
	for i=1,gridSize do
		for j=1,#numr[i] do
			love.graphics.print(numr[i][#numr[i]+1-j],gx-32-((j-1)*32),gy+((i-1)*(gs/gridSize))+(gs/gridSize/2-10))
		end
	end
end

function Debug()
	love.graphics.setFont(debugFont)
	
	if showDebug then
		love.graphics.print(mx..", "..my,0,60)
			
		for i=1,gridSize do
			for j=1,gridSize do
				love.graphics.print(grid[i][j],tw/2+gx-24+((i-1)*16),gy+((j-1)*16))
			end
		end
		
		for i=1,gridSize do
			for j=1,gridSize do
				love.graphics.print(player[i][j],tw/2+gx-24+((i-1)*16),th-gy-48+((j-1)*16))
			end
		end
		
		love.graphics.print(gs/gridSize,0,76)
	end
end